import greenfoot.Color;
import greenfoot.Greenfoot;
import greenfoot.GreenfootImage;
import greenfoot.World;
import packages.ui.*;


public class MyWorld extends UIWorld {

    Painter painter;
    Painting painting;

    final ColorPanel colorPanel;
    final PainterPanel painterPanel;

    public MyWorld(){
        super(600, 400, 1);
        GreenfootImage backgorund = new GreenfootImage("images/background.jpg");
        backgorund.scale(600, 400);
        setBackground(backgorund);
        addFps();
        Greenfoot.setSpeed(100);
        Greenfoot.start();

        add((colorPanel = new ColorPanel()), 1, 0.5, -75, 0);
        add((painterPanel = new PainterPanel()), 0.5, 0, 0, 40);

        add(painting = new Painting(400, 300), 0.379, 0.61);
        setPainter(new CirclePainter());

    }


    public Color getColor() {
        return new Color(
            colorPanel.red.getIntValue(),
            colorPanel.green.getIntValue(),
            colorPanel.blue.getIntValue(),
            colorPanel.alpha.getIntValue()
        );
    }

    public int getSize() {
        return colorPanel.size.getIntValue();
    }

    public void clearImage() {
        painting.clear();
    }

    public void setPainter(Painter painter) {
        if(this.painter != null) removeObject(this.painter);
        this.painter = painter;
        painter.setPainting(painting);
        addObject(painter, 0, 0);
    }






    class PainterPanel extends UIPanel {

        public PainterPanel() {
            super(590, 70, Color.DARK_GRAY);

            add(new Button("Circle").addClickAction(info -> setPainter(new CirclePainter())), 0.6, 0.3);
            add(new Button("Square").addClickAction(info -> setPainter(new SquarePainter())), 0.6, 0.7);
        }
    }




    class ColorPanel extends UIPanel {
    
        final Slider size, red, green, blue, alpha;
        final ColorSelector colSel;

        public ColorPanel() {
            super(140, 390, Color.DARK_GRAY);

            add(colSel = new ColorSelector(), 0.5, 0.2);
            size = new Slider(1, 100, 100);
            size.allowFractions(false);
            add(size, 0.5, 0.3);
            add(new Text("Size", 15, Color.LIGHT_GRAY), 0.5, 0.3, 0, 15);
            red = new Slider(0, 255, 100);
            red.allowFractions(false);
            add(red, 0.5, 0.38);
            add(new Text("Red", 15, Color.LIGHT_GRAY), 0.5, 0.38, 0, 15);
            green = new Slider(0, 255, 100);
            green.allowFractions(false);
            add(green, 0.5, 0.46);
            add(new Text("Green", 15, Color.LIGHT_GRAY), 0.5, 0.46, 0, 15);
            blue = new Slider(0, 255, 100);
            blue.allowFractions(false);
            add(blue, 0.5, 0.54);
            add(new Text("Blue", 15, Color.LIGHT_GRAY), 0.5, 0.54, 0, 15);
            alpha = new Slider(0, 255, 100);
            alpha.allowFractions(false);
            add(alpha, 0.5, 0.62);
            add(new Text("Transparency", 15, Color.LIGHT_GRAY), 0.5, 0.62, 0, 15);
            add(new Button("Clear").addClickAction(info -> clearImage()), 0.25, 0.95);
            add(new Button("Save").addClickAction(info -> Saver.save(painting.getImage())), 0.75, 0.95);

            add(new ColorPrefab(Color.RED), 0.5, 0.82, -40, -20);
            add(new ColorPrefab(Color.ORANGE), 0.5, 0.82, -20, -20);
            add(new ColorPrefab(Color.YELLOW), 0.5, 0.82, 0, -20);
            add(new ColorPrefab(Color.GREEN), 0.5, 0.82, 20, -20);
            add(new ColorPrefab(Color.CYAN), 0.5, 0.82, 40, -20);
            add(new ColorPrefab(Color.LIGHT_GRAY), 0.5, 0.82, -40, 0);
            add(new ColorPrefab(Color.WHITE), 0.5, 0.82, -20, 0);
            add(new ColorPrefab(Color.PINK), 0.5, 0.82, 0, 0);
            add(new ColorPrefab(Color.MAGENTA), 0.5, 0.82, 20, 0);
            add(new ColorPrefab(Color.BLUE), 0.5, 0.82, 40, 0);
            add(new ColorPrefab(Color.GRAY), 0.5, 0.82, -40, 20);
            add(new ColorPrefab(Color.DARK_GRAY), 0.5, 0.82, -20, 20);
            add(new ColorPrefab(Color.BLACK), 0.5, 0.82, 0, 20);
            add(new ColorPrefab(new Color(255, 202, 24)), 0.5, 0.82, 20, 20);
            add(new ColorPrefab(new Color(14, 209, 69)), 0.5, 0.82, 40, 20);
        }

        @Override
        protected void addedToWorld(World w) {
            super.addedToWorld(w);
            red.setValue(0);
            green.setValue(0);
            blue.setValue(0);
            alpha.setValue(255);
        }

        @Override
        public void act() {
            if(painter != null) {
                if(colSel.getClickCount() % 2 == 0) {
                    painter.setColor1(new Color(
                        colorPanel.red.getIntValue(),
                        colorPanel.green.getIntValue(),
                        colorPanel.blue.getIntValue(),
                        colorPanel.alpha.getIntValue()
                    ));
                }
                else{
                    painter.setColor2(new Color(
                        colorPanel.red.getIntValue(),
                        colorPanel.green.getIntValue(),
                        colorPanel.blue.getIntValue(),
                        colorPanel.alpha.getIntValue()
                    ));
                }
            }
        }


        class ColorPrefab extends Button {
            public ColorPrefab(Color color) {
                super(20, 20, color);
                addClickAction(info -> {
                    red.setValue(color.getRed());
                    green.setValue(color.getGreen());
                    blue.setValue(color.getBlue());
                    alpha.setValue(color.getAlpha());
                });
            }
        }

        class ColorSelector extends Button {
            public ColorSelector() {
                super(new Text("Editing color 1", Color.LIGHT_GRAY, new Color(0,0,0,0)), false);
                addClickAction(info -> {
                    if(clickCount % 2 == 0) {
                        getText().setContent("Editing color 1");
                        red.setValue(painter.getColor1().getRed());
                        green.setValue(painter.getColor1().getGreen());
                        blue.setValue(painter.getColor1().getBlue());
                        alpha.setValue(painter.getColor1().getAlpha());
                    }
                    else {
                        getText().setContent("Editing color 2");
                        red.setValue(painter.getColor2().getRed());
                        green.setValue(painter.getColor2().getGreen());
                        blue.setValue(painter.getColor2().getBlue());
                        alpha.setValue(painter.getColor2().getAlpha());
                    }
                });
            }
        }
    }
}