import greenfoot.*;

public abstract class Painter extends Actor {
    
    private Color color1 = Color.DARK_GRAY, color2 = Color.WHITE;

    private Painting painting;
    private int lastPaintSize = -1;

    protected ChangePreview changePreview;
    private Color current;
    private int alpha;


    public Painter(Painting painting) {
        this.painting = painting;
    }

    @Override
    protected void addedToWorld(World world) {
        updateImage(getSize());
    }

    public void startPainting(Color color) {
        if(painting == null || color == null || current != null) return;
        current = new Color(
            color.getRed(),
            color.getGreen(),
            color.getBlue()
        );
        alpha = color.getAlpha();
        getWorld().addObject(changePreview = new ChangePreview(), painting.getX(), painting.getY());
    }

    public void endPainting() {
        if(painting == null) return;
        paintPreviewOntoPainting();
        getWorld().removeObject(changePreview);
        alpha = 0;
        current = null;
    }

    private void paintPreviewOntoPainting() {
        for(int x=0; x<painting.getImage().getWidth(); x++) for(int y=0; y<painting.getImage().getHeight(); y++) {
            Color preview = changePreview.getImage().getColorAt(x, y);
            if(preview.getAlpha() == 0) continue;
            painting.getImage().setColor(new Color(
                preview.getRed(),
                preview.getGreen(),
                preview.getBlue(),
                alpha
            ));
            painting.getImage().fillRect(x, y, 1, 1);
        }
    }

    public void setPainting(Painting painting) {
        this.painting = painting;
    }

    public Painting getPainting() {
        return painting;
    }

    @Override
    public final void act() {
        updateImage();
        changePaintState();
        paint();
        moveToMouse();
    }


    private void paint() {
        if(current == null) return;
        try{
            MouseInfo mouse = Greenfoot.getMouseInfo();
            paint(mouse.getX(), mouse.getY());
        }
        catch(Exception e) {
            e.printStackTrace(System.out);
            endPainting();
        }
    }


    private void updateImage() {
        if(getSize() != lastPaintSize) {
            lastPaintSize = getSize();
            updateImage(lastPaintSize);
        }
    }

    public abstract void updateImage(int size);


    private void moveToMouse() {
        try{
            MouseInfo mouse = Greenfoot.getMouseInfo();
            setLocation(mouse.getX(), mouse.getY());
        }catch(Exception e) {}
    }


    private void changePaintState() {
        if(painting == null) return;
        if(Greenfoot.mousePressed(null)) {
            try{
                MouseInfo mouse = Greenfoot.getMouseInfo();
                if(mouse.getButton() == 1) startPainting(color1);
                else if(mouse.getButton() == 3) startPainting(color2);
            }catch(Exception e) {}
        }
        else if(Greenfoot.mouseClicked(null)) endPainting();
    }

    private void paint(int mouseX, int mouseY) {
        int startX = getX() + painting.getImage().getWidth() / 2 - painting.getX();
        int startY = getY() + painting.getImage().getHeight() / 2 - painting.getY();
        int endX = mouseX + painting.getImage().getWidth() / 2 - painting.getX();
        int endY = mouseY + painting.getImage().getHeight() / 2 - painting.getY();
        paint(startX, startY, endX, endY, getSize(), current, changePreview.getImage());
    }

    public void setColor1(Color color) {
        if(color == null) return;
        color1 = color;
    }

    public void setColor2(Color color) {
        if(color == null) return;
        color2 = color;
    }

    public Color getColor1() {
        return color1;
    }

    public Color getColor2() {
        return color2;
    }

    public int getSize() {
        return ((MyWorld)getWorld()).getSize();
    }

    public abstract void paint(int startX, int startY, int endX, int endY, int size, Color color, GreenfootImage image);




    class ChangePreview extends Actor {
        public ChangePreview() {
            setImage(new GreenfootImage(painting.getImage().getWidth(), painting.getImage().getHeight()));
            getImage().setTransparency(alpha);
        }
        @Override
        protected void addedToWorld(World world) {
            world.setPaintOrder(ChangePreview.class);
        }
    }
}